@@ -31,7 +31,7 @@ struct st_mysqlx_column_result : public util::custom_allocable
 	const drv::st_xmysqlnd_result_field_meta* meta;
 };
 
-void mysqlx_new_column_result(zval* return_value, const drv::st_xmysqlnd_result_field_meta * meta);
+util::zvalue create_column_result(const drv::st_xmysqlnd_result_field_meta* meta);
 void mysqlx_register_column_result_class(INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers);
 void mysqlx_unregister_column_result_class(SHUTDOWN_FUNC_ARGS);
 
