@@ -38,7 +38,7 @@ struct st_mysqlx_doc_result_iterator : util::custom_allocable
 {
 	zend_object_iterator  intern;
 	XMYSQLND_STMT_RESULT * result;
-	zval current_row;
+	util::zvalue current_row;
 	size_t row_num;
 	zend_bool started;
 	zend_bool usable;
@@ -56,7 +56,6 @@ XMYSQLND_METHOD(mysqlx_doc_result_iterator, dtor)(zend_object_iterator * iter)
 
 	/* cleanup handled in sxe_object_dtor as we dont always have an iterator wrapper */
 	zval_ptr_dtor(&iterator->intern.data);
-	zval_ptr_dtor(&iterator->current_row);
 	DBG_VOID_RETURN;
 }
 
@@ -71,33 +70,29 @@ XMYSQLND_METHOD(mysqlx_doc_result_iterator, valid)(zend_object_iterator * iter)
 
 #include <ext/standard/php_var.h>
 
-static zval *
+static util::raw_zval*
 XMYSQLND_METHOD(mysqlx_doc_result_iterator, current_data)(zend_object_iterator * iter)
 {
 	st_mysqlx_doc_result_iterator* iterator = (st_mysqlx_doc_result_iterator*) iter;
 	DBG_ENTER("mysqlx_doc_result_iterator::current_data");
 	DBG_INF_FMT("usable=%s  started=%s  row_num=%u", iterator->usable? "TRUE":"FALSE", iterator->started? "TRUE":"FALSE", iterator->row_num);
-	DBG_RETURN((iterator->result && iterator->usable)? &iterator->current_row : nullptr);
+	DBG_RETURN((iterator->result && iterator->usable)? iterator->current_row.ptr() : nullptr);
 }
 
 static enum_func_status
 XMYSQLND_METHOD(mysqlx_doc_result_iterator, fetch_current_data)(zend_object_iterator * iter)
 {
-	st_mysqlx_doc_result_iterator* iterator = (st_mysqlx_doc_result_iterator*) iter;
+	st_mysqlx_doc_result_iterator* iterator = reinterpret_cast<st_mysqlx_doc_result_iterator*>(iter);
 	DBG_ENTER("mysqlx_doc_result_iterator::fetch_current_data");
 	DBG_INF_FMT("usable=%s  started=%s  row_num=%u", iterator->usable? "TRUE":"FALSE", iterator->started? "TRUE":"FALSE", iterator->row_num);
 	if (iterator->result && iterator->usable) {
-		zval_ptr_dtor(&iterator->current_row);
-		ZVAL_UNDEF(&iterator->current_row);
+		iterator->current_row.reset();
 
-		zval current_row;
-		ZVAL_UNDEF(&current_row);
-
-		if (PASS == iterator->result->m.fetch_current(iterator->result, &current_row, nullptr, nullptr) &&
-			IS_ARRAY == Z_TYPE(current_row))
+		util::zvalue current_row;
+		if (PASS == iterator->result->m.fetch_current(iterator->result, current_row.ptr(), nullptr, nullptr) &&
+			 current_row.is_array())
 		{
-			xmysqlnd_utils_decode_doc_row(&current_row, &iterator->current_row);
-			zval_ptr_dtor(&current_row);
+			iterator->current_row = xmysqlnd_utils_decode_doc_row(current_row);
 			DBG_RETURN(PASS);
 		} else {
 			DBG_RETURN(FAIL);
@@ -156,8 +151,8 @@ static zend_object_iterator_funcs mysqlx_doc_result_iterator_funcs =
 	XMYSQLND_METHOD(mysqlx_doc_result_iterator, rewind),
 };
 
-static zend_object_iterator *
-mysqlx_doc_result_create_iterator(zend_class_entry * ce, zval * object, int by_ref)
+static zend_object_iterator*
+mysqlx_doc_result_create_iterator(zend_class_entry* ce, util::raw_zval* object, int by_ref)
 {
 	DBG_ENTER("mysqlx_doc_result_create_iterator");
 	auto iterator = util::create_result_iterator<st_mysqlx_doc_result, st_mysqlx_doc_result_iterator>(
