@@ -50,7 +50,7 @@ st_mysqlx_object* mysqlx_fetch_object_from_zo(zend_object *obj);
 #define Z_MYSQLX_P(zv) mysqlx::devapi::mysqlx_fetch_object_from_zo(Z_OBJ_P((zv)))
 
 void mysqlx_object_free_storage(zend_object * object);
-HashTable * mysqlx_object_get_debug_info(zval *object, int *is_temp);
+HashTable * mysqlx_object_get_debug_info(util::raw_zval* object, int* is_temp);
 
 } // namespace devapi
 
