@@ -34,7 +34,7 @@
 #include "mysqlx_base_result.h"
 #include "util/object.h"
 #include "util/string_utils.h"
-#include "util/zend_utils.h"
+#include "util/functions.h"
 
 namespace mysqlx {
 
@@ -72,8 +72,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, __construct)
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAffectedItemsCount)
 {
 	DBG_ENTER("mysqlx_result::getAffectedItemsCount");
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -86,12 +86,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAffectedItemsCount)
 	const XMYSQLND_STMT_EXECUTION_STATE* exec_state = data_object.result->exec_state;
 	/* Maybe check here if there was an error and throw an Exception or return a warning */
 	if (exec_state) {
-		const size_t value = exec_state->m->get_affected_items_count(exec_state);
+		const uint64_t value = exec_state->m->get_affected_items_count(exec_state);
 		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
 			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
 			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
 		} else {
-			RETVAL_LONG(value);
+			RETVAL_LONG(static_cast<zend_long>(value));
 			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
@@ -101,8 +101,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAffectedItemsCount)
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAutoIncrementValue)
 {
 	DBG_ENTER("mysqlx_result::getAutoIncrementValue");
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -131,8 +131,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAutoIncrementValue)
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getGeneratedIds)
 {
     DBG_ENTER("mysqlx_result::getGeneratedIds");
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -158,8 +158,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getGeneratedIds)
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getWarningsCount)
 {
 	DBG_ENTER("mysqlx_result::getWarningsCount");
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -185,8 +185,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getWarningsCount)
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getWarnings)
 {
 	DBG_ENTER("mysqlx_result::getWarnings");
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -200,16 +200,15 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getWarnings)
 		array_init_size(return_value, static_cast<uint32_t>(count));
 		for (std::size_t i{0}; i < count; ++i) {
 			const XMYSQLND_WARNING warning = warnings->get_warning(i);
-			util::zvalue warning_zv;
-			mysqlx_new_warning(warning_zv.ptr(), warning.message, warning.level, warning.code);
-			if (!warning_zv.is_undef()) {
-				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), warning_zv.ptr());
-				warning_zv.invalidate();
+			util::zvalue warning_obj = create_warning(warning.message, warning.level, warning.code);
+			if (!warning_obj.is_undef()) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), warning_obj.ptr());
+				warning_obj.invalidate();
 			}
 		}
 	}
 
-	util::zend::ensure_is_array(return_value);
+	util::zvalue::ensure_is_array(return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -274,16 +273,17 @@ mysqlx_unregister_result_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 	zend_hash_destroy(&mysqlx_result_properties);
 }
 
-void
-mysqlx_new_result(zval* return_value, XMYSQLND_STMT_RESULT* result)
+util::zvalue
+create_result(XMYSQLND_STMT_RESULT* result)
 {
-	DBG_ENTER("mysqlx_new_result");
+	DBG_ENTER("create_result");
 
+	util::zvalue result_obj;
 	st_mysqlx_result& data_object{
-		util::init_object<st_mysqlx_result>(mysqlx_result_class_entry, return_value) };
+		util::init_object<st_mysqlx_result>(mysqlx_result_class_entry, result_obj) };
 	data_object.result = result;
 
-	DBG_VOID_RETURN;
+	DBG_RETURN(result_obj);
 }
 
 } // namespace devapi
