@@ -18,6 +18,7 @@
 #include "php_api.h"
 #include "mysqlx_database_object.h"
 #include "mysqlx_schema_object.h"
+#include "util/object.h"
 
 namespace mysqlx {
 
@@ -39,10 +40,11 @@ mysqlx_register_schema_object_interface(
 	UNUSED_INIT_FUNC_ARGS,
 	zend_object_handlers* /*mysqlx_std_object_handlers*/)
 {
-	zend_class_entry tmp_ce;
-	INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "SchemaObject", mysqlx_schema_object_methods);
-	mysqlx_schema_object_interface_entry = zend_register_internal_interface(&tmp_ce);
-	zend_class_implements(mysqlx_schema_object_interface_entry, 1, mysqlx_database_object_interface_entry);
+	MYSQL_XDEVAPI_REGISTER_INTERFACE(
+		mysqlx_schema_object_interface_entry,
+		"SchemaObject",
+		mysqlx_schema_object_methods,
+		mysqlx_database_object_interface_entry);
 }
 
 void
