@@ -34,8 +34,8 @@ struct Session_data : public util::custom_allocable
 	bool close_connection();
 };
 
-void mysqlx_new_session(zval* return_value);
-void mysqlx_new_session(zval* return_value, drv::XMYSQLND_SESSION session);
+util::zvalue create_session();
+util::zvalue create_session(drv::XMYSQLND_SESSION session);
 void mysqlx_register_session_class(INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers);
 void mysqlx_unregister_session_class(SHUTDOWN_FUNC_ARGS);
 
