@@ -64,27 +64,20 @@ struct st_mysqlx_statement : public util::custom_allocable
 void mysqlx_register_statement_class(INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers);
 void mysqlx_unregister_statement_class(SHUTDOWN_FUNC_ARGS);
 
-void mysqlx_new_stmt(zval* return_value, drv::xmysqlnd_stmt* stmt);
-void mysqlx_statement_execute_read_response(
+util::zvalue create_stmt(drv::xmysqlnd_stmt* stmt);
+util::zvalue mysqlx_statement_execute_read_response(
 	const st_mysqlx_object* const mysqlx_object,
 	const zend_long flags,
-	const mysqlx_result_type result_type,
-	zval* return_value);
-
-void execute_new_statement_read_response(
-	drv::xmysqlnd_stmt* stmt,
-	const zend_long flags,
-	const mysqlx_result_type result_type,
-	zval* return_value);
+	const mysqlx_result_type result_type);
 
 /**********/
 
 void mysqlx_register_sql_statement_class(INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers);
 void mysqlx_unregister_sql_statement_class(SHUTDOWN_FUNC_ARGS);
 
-void mysqlx_new_sql_stmt(zval* return_value, drv::xmysqlnd_stmt* stmt, const std::string_view& namespace_, const util::string_view& query);
-void mysqlx_sql_statement_bind_one_param(zval * object_zv, const zval * param_zv, zval* return_value);
-void mysqlx_sql_statement_execute(const st_mysqlx_object* const mysqlx_object, const zend_long flags, zval* return_value);
+util::zvalue create_sql_stmt(drv::xmysqlnd_stmt* stmt, const std::string_view& namespace_, const util::string_view& query);
+bool mysqlx_sql_statement_bind_one_param(util::raw_zval* object_zv, const util::zvalue& param);
+util::zvalue mysqlx_sql_statement_execute(const st_mysqlx_object* const mysqlx_object, const zend_long flags);
 
 } // namespace devapi
 
