@@ -33,8 +33,8 @@
 #include "mysqlx_table__update.h"
 #include "mysqlx_table.h"
 #include "util/allocator.h"
+#include "util/functions.h"
 #include "util/object.h"
-#include "util/zend_utils.h"
 
 namespace mysqlx {
 
@@ -109,8 +109,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSession)
 {
 	DBG_ENTER("mysqlx_table::getSession");
 
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(
 		execute_data,
 		getThis(),
 		"O",
@@ -125,7 +125,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSession)
 	RETVAL_FALSE;
 
 	XMYSQLND_SESSION session{ data_object.table->get_schema()->get_session() };
-	mysqlx_new_session(return_value, session);
+	create_session(session).move_to(return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -134,8 +134,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getName)
 {
 	DBG_ENTER("mysqlx_table::getName");
 
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -162,7 +162,7 @@ mysqlx_table_on_error(
 	if (code == UnknownDatabaseCode) {
 		DBG_RETURN(HND_PASS);
 	} else {
-		mysqlx_new_exception(code, sql_state, message);
+		create_exception(code, sql_state, message);
 		DBG_RETURN(HND_PASS_RETURN_FAIL);
 	}
 }
@@ -171,8 +171,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, existsInDatabase)
 {
 	DBG_ENTER("mysqlx_table::existsInDatabase");
 
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -197,8 +197,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, isView)
 {
 	DBG_ENTER("mysqlx_table::isView");
 
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(
 		execute_data,
 		getThis(),
 		"O",
@@ -227,8 +227,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, count)
 {
 	DBG_ENTER("mysqlx_table::count");
 
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -255,8 +255,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSchema)
 	DBG_ENTER("mysqlx_table::getSchema");
 
 	XMYSQLND_SESSION session;
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(
 				execute_data,
 				getThis(), "O",
 				&object_zv,
@@ -275,7 +275,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSchema)
 		xmysqlnd_schema* schema = session->create_schema_object(
 			data_object.table->get_schema()->get_name());
 		if (schema) {
-			mysqlx_new_schema(return_value, schema);
+			create_schema(schema).move_to(return_value);
 		} else {
 			RAISE_EXCEPTION(10001,"Invalid object of class schema");
 		}
@@ -292,10 +292,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, insert)
 {
 	DBG_ENTER("mysqlx_table::insert");
 
-	zval* object_zv{nullptr};
+	util::raw_zval* object_zv{nullptr};
 	zval* columns{nullptr};
 	int num_of_columns{0};
-	if (FAILURE == util::zend::parse_method_parameters(
+	if (FAILURE == util::get_method_arguments(
 		execute_data, getThis(), "O+",
 		&object_zv, mysqlx_table_class_entry,
 		&columns,
@@ -323,10 +323,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, insert)
 	RETVAL_FALSE;
 
 	if (num_of_columns > 0) {
-		mysqlx_new_table__insert(return_value,
-									  data_object.table,
-									  columns,
-									  num_of_columns);
+		create_table_insert(
+			data_object.table,
+			columns,
+			num_of_columns)
+			.move_to(return_value);
 	}
 
 	DBG_VOID_RETURN;
@@ -336,10 +337,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, select)
 {
 	DBG_ENTER("mysqlx_table::select");
 
-	zval* object_zv{nullptr};
+	util::raw_zval* object_zv{nullptr};
 	zval* columns{nullptr};
 	int num_of_columns{0};
-	if (FAILURE == util::zend::parse_method_parameters(
+	if (FAILURE == util::get_method_arguments(
 		execute_data, getThis(), "O+",
 		&object_zv, mysqlx_table_class_entry,
 		&columns,
@@ -355,10 +356,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, select)
 	if (columns) {
 		DBG_INF_FMT("Num of columns: %d",
 					num_of_columns);
-		mysqlx_new_table__select(return_value,
-						data_object.table,
-						columns,
-						num_of_columns);
+		create_table_select(
+			data_object.table,
+			columns,
+			num_of_columns)
+			.move_to(return_value);
 	}
 
 	DBG_VOID_RETURN;
@@ -368,8 +370,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, update)
 {
 	DBG_ENTER("mysqlx_table::update");
 
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -379,7 +381,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, update)
 
 	RETVAL_FALSE;
 
-	mysqlx_new_table__update(return_value, data_object.table);
+	create_table_update(data_object.table).move_to(return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -388,8 +390,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, delete)
 {
 	DBG_ENTER("mysqlx_table::delete");
 
-	zval* object_zv{nullptr};
-	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
+	util::raw_zval* object_zv{nullptr};
+	if (FAILURE == util::get_method_arguments(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
@@ -399,7 +401,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, delete)
 
 	RETVAL_FALSE;
 
-	mysqlx_new_table__delete(return_value, data_object.table);
+	create_table_delete(data_object.table).move_to(return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -424,8 +426,8 @@ static const zend_function_entry mysqlx_table_methods[] = {
 	{nullptr, nullptr, nullptr}
 };
 
-static zval *
-mysqlx_table_property__name(const st_mysqlx_object* obj, zval* return_value)
+static util::raw_zval*
+mysqlx_table_property__name(const st_mysqlx_object* obj, util::raw_zval* return_value)
 {
 	const st_mysqlx_table* object = (const st_mysqlx_table* ) (obj->ptr);
 	DBG_ENTER("mysqlx_table_property__name");
@@ -495,14 +497,15 @@ mysqlx_unregister_table_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 	zend_hash_destroy(&mysqlx_table_properties);
 }
 
-void
-mysqlx_new_table(zval* return_value, xmysqlnd_table* table)
+util::zvalue
+create_table(xmysqlnd_table* table)
 {
-	DBG_ENTER("mysqlx_new_table");
+	DBG_ENTER("create_table");
+	util::zvalue table_obj;
 	st_mysqlx_table& data_object{
-		util::init_object<st_mysqlx_table>(mysqlx_table_class_entry, return_value) };
+		util::init_object<st_mysqlx_table>(mysqlx_table_class_entry, table_obj) };
 	data_object.table = table;
-	DBG_VOID_RETURN;
+	DBG_RETURN(table_obj);
 }
 
 } // namespace devapi
