@@ -71,4 +71,10 @@ extern "C" {
 
 #endif
 
+namespace mysqlx::util {
+
+using raw_zval = zval;
+
+}
+
 #endif // MYSQL_XDEVAPI_PHP_API_H
