@@ -27,11 +27,14 @@
 #include "proto_gen/mysqlx_expect.pb.h"
 
 #include "util/strings.h"
+#include "util/value.h"
 
 #include <optional>
 
 namespace mysqlx {
 
+namespace util { class zvalue; }
+
 namespace drv {
 
 namespace compression { class Executor; }
@@ -51,6 +54,7 @@ struct Message_context
 	MYSQLND_STATS* stats;
 	MYSQLND_ERROR_INFO* error_info;
 	compression::Executor* compression_executor;
+	Session_callback* session_callback;
 };
 
 
@@ -70,31 +74,30 @@ struct st_xmysqlnd_on_error_bind
 
 struct st_xmysqlnd_on_session_var_change_bind
 {
-	const enum_hnd_func_status (*handler)(void * context, const util::string_view& name, const zval * value);
+	const enum_hnd_func_status (*handler)(void * context, const util::string_view& name, const util::zvalue& value);
 	void * ctx;
 };
 
-
 struct st_xmysqlnd_msg__capabilities_get
 {
 	enum_func_status (*send_request)(st_xmysqlnd_msg__capabilities_get* msg);
 
 	enum_func_status (*read_response)(st_xmysqlnd_msg__capabilities_get* msg,
-									  zval * capabilities);
+		util::zvalue* capabilities);
 
 	enum_func_status (*init_read)(st_xmysqlnd_msg__capabilities_get* const msg,
 								  const st_xmysqlnd_on_error_bind on_error);
 
 	Message_context msg_ctx;
 	st_xmysqlnd_on_error_bind on_error;
-	zval* capabilities_zval;
+	util::zvalue* capabilities;
 };
 
 
 struct st_xmysqlnd_msg__capabilities_set
 {
 	enum_func_status (*send_request)(st_xmysqlnd_msg__capabilities_set* msg,
-									 const size_t cap_count, zval ** capabilities_names, zval ** capabilities_values);
+		const util::zvalue& capabilities);
 
 	enum_func_status (*read_response)(st_xmysqlnd_msg__capabilities_set* msg,
 									  zval* return_value);
@@ -115,7 +118,7 @@ struct st_xmysqlnd_on_auth_continue_bind
 
 struct st_xmysqlnd_on_client_id_bind
 {
-	enum_func_status (*handler)(void * context, const size_t id);
+	enum_func_status (*handler)(void * context, const uint64_t id);
 	void * ctx;
 };
 
@@ -168,7 +171,7 @@ struct st_xmysqlnd_on_meta_field_bind
 
 struct st_xmysqlnd_on_execution_state_change_bind
 {
-	const enum_hnd_func_status (*handler)(void * context, const enum xmysqlnd_execution_state_type type, const size_t value);
+	const enum_hnd_func_status (*handler)(void * context, const enum xmysqlnd_execution_state_type type, const uint64_t value);
 	void * ctx;
 };
 
